

set -ex



csvclean --help
csvcut --help
csvformat --help
csvgrep --help
csvjoin --help
csvjson --help
csvlook --help
csvpy --help
csvsort --help
csvsql --help
csvstack --help
csvstat --help
in2csv --help
sql2csv --help
exit 0
